"""
Maintain version for this package.
Do not edit this file, use 'version' section of config.
"""
# {# pkglts, version
#  -*- coding: utf-8 -*-

MAJOR = {{ version.major }}
"""(int) Version major component."""

MINOR = {{ version.minor }}
"""(int) Version minor component."""

POST = {{ version.post }}
"""(int) Version post or bugfix component."""

__version__ = ".".join([str(s) for s in (MAJOR, MINOR, POST)])
# #}
